@@ -20,14 +20,14 @@
  */
 /*
   File:      actions.cpp
-  Version:   $Rev: 1366 $
+  Version:   $Rev: 1458 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
              30-Apr-06, Roger Larsson: Print filename if processing multiple files
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: actions.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: actions.cpp 1458 2008-05-22 08:38:20Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -44,6 +44,7 @@ EXIV2_RCSID("@(#) $Id: actions.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 #include "exiv2.hpp"
 #include "image.hpp"
 #include "jpgimage.hpp"
+#include "xmpsidecar.hpp"
 #include "utils.hpp"
 #include "types.hpp"
 #include "exif.hpp"
@@ -112,12 +113,15 @@ namespace {
       @param source Source file path
       @param target Target file path. An *.exv file is created if target doesn't
                     exist.
+      @param targetType Image type for the target image in case it needs to be
+                    created.
       @param preserve Indicates if existing metadata in the target file should
                     be kept.
       @return 0 if successful, else an error code
     */
     int metacopy(const std::string& source,
                  const std::string& target,
+                 int targetType,
                  bool preserve);
 
     /*!
@@ -241,11 +245,6 @@ namespace Action {
         assert(image.get() != 0);
         image->readMetadata();
         Exiv2::ExifData& exifData = image->exifData();
-        if (exifData.empty()) {
-            std::cerr << path_ << ": "
-                      << _("No Exif data found in the file\n");
-            return -3;
-        }
         align_ = 16;
 
         // Filename
@@ -259,6 +258,20 @@ namespace Action {
             std::cout << buf.st_size << " " << _("Bytes") << std::endl;
         }
 
+        // MIME type 
+        printLabel(_("MIME type"));
+        std::cout << image->mimeType() << "\n";
+
+        // Image size
+        printLabel(_("Image size"));
+        std::cout << image->pixelWidth() << " x " << image->pixelHeight() << "\n";
+
+        if (exifData.empty()) {
+            std::cerr << path_ << ": "
+                      << _("No Exif data found in the file\n");
+            return -3;
+        }
+
         // Camera make
         printTag(exifData, "Exif.Image.Make", _("Camera make"));
 
@@ -775,7 +788,7 @@ namespace Action {
                       << std::dec << std::setw(3)
                       << std::setfill(' ') << std::right
                       << md->count() << "  "
-                      << std::dec << md->value()
+                      << std::dec << *md
                       << std::endl;
         }
 
@@ -1011,13 +1024,16 @@ namespace Action {
         if (Params::instance().target_ & Params::ctThumb) {
             rc = writeThumbnail();
         }
-        if (Params::instance().target_ & Params::ctXmpPacket) {
-            rc = writeXmp();
+        if (Params::instance().target_ & Params::ctXmpSidecar) {
+            std::string xmpPath = newFilePath(path_, ".xmp");
+            if (dontOverwrite(xmpPath)) return 0;
+            rc = metacopy(path_, xmpPath, Exiv2::ImageType::xmp, false);
         }
-        if (Params::instance().target_ & ~Params::ctThumb & ~Params::ctXmpPacket) {
+        if (   !(Params::instance().target_ & Params::ctXmpSidecar)
+            && !(Params::instance().target_ & Params::ctThumb)) {
             std::string exvPath = newFilePath(path_, ".exv");
             if (dontOverwrite(exvPath)) return 0;
-            rc = metacopy(path_, exvPath, false);
+            rc = metacopy(path_, exvPath, Exiv2::ImageType::exv, false);
         }
         return rc;
     }
@@ -1028,37 +1044,6 @@ namespace Action {
         return 1;
     } // Extract::run
 
-    int Extract::writeXmp() const
-    {
-        if (!Exiv2::fileExists(path_, true)) {
-            std::cerr << path_
-                      << ": " << _("Failed to open the file\n");
-            return -1;
-        }
-        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path_);
-        assert(image.get() != 0);
-        image->readMetadata();
-        const std::string& xmpPacket = image->xmpPacket();
-        if (xmpPacket.empty()) {
-            return -3;
-        }
-        std::string xmpPath = newFilePath(path_, ".xmp");
-        if (dontOverwrite(xmpPath)) return 0;
-        if (Params::instance().verbose_) {
-            std::cout << _("Writing XMP packet from") << " " << path_
-                      << " " << _("to") << " " << xmpPath << std::endl;
-        }
-        std::ofstream file(xmpPath.c_str());
-        if (!file) {
-            std::cerr << Params::instance().progname() << ": " 
-                      << _("Failed to open file ") << " " << xmpPath << ": "
-                      << Exiv2::strError() << "\n";
-            return 1;
-        }
-        file << xmpPacket;
-        return 0;
-    } // Extract::writeXmp
-
     int Extract::writeThumbnail() const
     {
         if (!Exiv2::fileExists(path_, true)) {
@@ -1125,16 +1110,17 @@ namespace Action {
             rc = insertThumbnail(path);
         }
         if (   rc == 0
-            && Params::instance().target_ & Params::ctExif
-            || Params::instance().target_ & Params::ctIptc
-            || Params::instance().target_ & Params::ctComment
-            || Params::instance().target_ & Params::ctXmp) {
+            && (   Params::instance().target_ & Params::ctExif
+                || Params::instance().target_ & Params::ctIptc
+                || Params::instance().target_ & Params::ctComment
+                || Params::instance().target_ & Params::ctXmp)) {
             std::string suffix = Params::instance().suffix_;
             if (suffix.empty()) suffix = ".exv";
+            if (Params::instance().target_ & Params::ctXmpSidecar) suffix = ".xmp";
             std::string exvPath = newFilePath(path, suffix);
-            rc = metacopy(exvPath, path, true);
+            rc = metacopy(exvPath, path, Exiv2::ImageType::none, true);
         }
-        if (0 == rc && Params::instance().target_ & Params::ctXmpPacket) {
+        if (0 == rc && Params::instance().target_ & Params::ctXmpSidecar) {
             rc = insertXmpPacket(path);
         }
         if (Params::instance().preserve_) {
@@ -1413,7 +1399,7 @@ namespace Action {
             std::cout << _("Reg ") << modifyCmd.key_ << "=\"" 
                       << modifyCmd.value_ << "\"" << std::endl;
         }
-        Exiv2::XmpProperties::registerNs(modifyCmd.value_, modifyCmd.key_);
+        // Registration has been done immediately after parsing the command.
     }
 
     Modify::AutoPtr Modify::clone() const
@@ -1734,6 +1720,7 @@ namespace {
 
     int metacopy(const std::string& source,
                  const std::string& target,
+                 int targetType,
                  bool preserve)
     {
         if (!Exiv2::fileExists(source, true)) {
@@ -1755,8 +1742,7 @@ namespace {
             if (preserve) targetImage->readMetadata();
         }
         else {
-            targetImage
-                = Exiv2::ImageFactory::create(Exiv2::ImageType::exv, target);
+            targetImage = Exiv2::ImageFactory::create(targetType, target);
             assert(targetImage.get() != 0);
         }
         if (   Params::instance().target_ & Params::ctExif
