@@ -20,7 +20,7 @@
  */
 /*
   File:      convert.cpp
-  Version:   $Rev: 1493 $
+  Version:   $Rev: 1499 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   17-Mar-08, ahu: created basic converter framework
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: convert.cpp 1493 2008-06-06 02:51:18Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: convert.cpp 1499 2008-06-10 03:22:59Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -548,15 +548,28 @@ namespace Exiv2 {
             }
         }
         else { // "Exif.GPSInfo.GPSTimeStamp"
+
+            bool ok = true;
+            if (pos->value().count() != 3) ok = false; 
+            if (ok) {
+                for (int i = 0; i < 3; ++i) {
+                    if (pos->value().toRational(i).second == 0) {
+                        ok = false;
+                        break;
+                    }
+                }
+            }
+            if (!ok) {
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
+#endif
+                return;
+            }
+
             double dhour = pos->value().toFloat(0);
             double dmin = pos->value().toFloat(1);
             // Hack: Need Value::toDouble
             URational r = pos->value().toRational(2);
-            if (r.second == 0) {
-#ifndef SUPPRESS_WARNINGS
-                std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
-#endif
-            }
             double dsec = static_cast<double>(r.first)/r.second;
 
             if (!pos->value().ok()) {
@@ -702,7 +715,7 @@ namespace Exiv2 {
         for (int i = 0; i < 3; ++i) {
             const int32_t z = pos->value().toRational(i).first;
             const int32_t d = pos->value().toRational(i).second;
-            if (d == 0.0) {
+            if (d == 0) {
 #ifndef SUPPRESS_WARNINGS
                 std::cerr << "Warning: Failed to convert " << from << " to " << to << "\n";
 #endif
@@ -1079,7 +1092,8 @@ namespace Exiv2 {
                 continue;
             }
             // Todo: Escape non-ASCII characters in XMP text values
-            Iptcdatum id(IptcKey(to), 0);
+            IptcKey key(to);
+            Iptcdatum id(key);
             id.setValue(value);
             iptcData_->add(id);
         }
