@@ -20,7 +20,7 @@
  */
 /*
   File:      jpgimage.cpp
-  Version:   $Rev: 1404 $
+  Version:   $Rev: 1503 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   15-Jan-05, brad: split out from image.cpp
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1404 2008-04-15 03:46:01Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1503 2008-06-14 13:41:48Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -405,11 +405,16 @@ namespace Exiv2 {
                          || marker == sof7_  || marker == sof9_  || marker == sof10_
                          || marker == sof11_ || marker == sof13_ || marker == sof14_
                          || marker == sof15_)) {
-                // we hit a SOFn (start-of-frame) marker
-                if (size < 8) throw Error(15);
+                // We hit a SOFn (start-of-frame) marker
+                if (size < 8) {
+                    rc = 7;
+                    break;
+                }
                 pixelHeight_ = getUShort(buf.pData_ + 3, bigEndian);
                 pixelWidth_ = getUShort(buf.pData_ + 5, bigEndian);
                 if (pixelHeight_ != 0) --search;
+                // Skip the remainder of the segment
+                io_->seek(size-bufRead, BasicIo::cur);
             }
             else {
                 if (size < 2) {
