@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version $Rev: 1366 $
+  @version $Rev: 1479 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -50,13 +50,15 @@ namespace Exiv2 {
 // class declarations
     class Value;
     class Entry;
+    struct TagInfo;
 
 // *****************************************************************************
 // type definitions
 
     //! Type for a function pointer for functions interpreting the tag value
     typedef std::ostream& (*PrintFct)(std::ostream&, const Value&);
-
+    //! A function returning a tag list.
+    typedef const TagInfo* (*TagListFct)();
     /*!
       @brief Section identifiers to logically group tags. A section consists
              of nothing more than a name, based on the Exif standard.
@@ -72,18 +74,24 @@ namespace Exiv2 {
 
     //! Contains information pertaining to one IFD
     struct IfdInfo {
-        //! Constructor
-        IfdInfo(IfdId ifdId, const char* name, const char* item);
+        struct Item;
+        bool operator==(IfdId ifdId) const;     //!< Comparison operator for IFD id
+        bool operator==(Item item) const;       //!< Comparison operator for IFD item
         IfdId ifdId_;                           //!< IFD id
         const char* name_;                      //!< IFD name
         //! Related IFD item. This is also an IFD name, unique for each IFD.
         const char* item_;
+        TagListFct tagList_;                    //!< Tag list
+    };
+
+    //! Search key to find an IfdInfo by its IFD item.
+    struct IfdInfo::Item {
+        Item(const std::string& item);          //!< Constructor
+        std::string i_;                         //!< IFD item
     };
 
     //! Contains information pertaining to one section
     struct SectionInfo {
-        //! Constructor
-        SectionInfo(SectionId sectionId, const char* name, const char* desc);
         SectionId sectionId_;                   //!< Section id
         const char* name_;                      //!< Section name (one word)
         const char* desc_;                      //!< Section description
@@ -282,37 +290,22 @@ namespace Exiv2 {
         static const TagInfo* gpsTagList();
         //! Print a list of all standard Exif tags to output stream
         static void taglist(std::ostream& os);
-        //! Print a list of all tags related to one makernote %IfdId
-        static void makerTaglist(std::ostream& os, IfdId ifdId);
-        //! Register an %IfdId with the base IFD %TagInfo list for a makernote
-        static void registerBaseTagInfo(IfdId ifdId);
+        //! Print the list of tags for \em %IfdId
+        static void taglist(std::ostream& os, IfdId ifdId);
         /*!
-          @brief Register an %IfdId and %TagInfo list for a makernote
-
-          @throw Error if the MakerTagInfo registry is full
-         */
-        static void registerMakerTagInfo(IfdId ifdId, const TagInfo* tagInfo);
-        /*!
-          @brief Return true if \em ifdId is an %Ifd Id which is registered
-                 as a makernote %Ifd id. Note: Calling this function with
+          @brief Return true if \em ifdId is an %Ifd id which is 
+                 a makernote %Ifd id. Note: Calling this function with
                  makerIfd returns false.
         */
         static bool isMakerIfd(IfdId ifdId);
 
     private:
-        static int tagInfoIdx(uint16_t tag, IfdId ifdId);
-        static const TagInfo* makerTagInfo(uint16_t tag, IfdId ifdId);
-        static const TagInfo* makerTagInfo(const std::string& tagName,
-                                           IfdId ifdId);
+        static const TagInfo* tagList(IfdId ifdId);
+        static const TagInfo* tagInfo(uint16_t tag, IfdId ifdId);
+        static const TagInfo* tagInfo(const std::string& tagName, IfdId ifdId);
 
-        static const IfdInfo     ifdInfo_[];
-        static const SectionInfo sectionInfo_[];
-
-        static const TagInfo*    tagInfos_[];
-
-        static const int         MAX_MAKER_TAG_INFOS = 64;
-        static const TagInfo*    makerTagInfos_[MAX_MAKER_TAG_INFOS];
-        static IfdId             makerIfdIds_[MAX_MAKER_TAG_INFOS];
+        static const IfdInfo     ifdInfo_[];     //!< All Exif and Makernote tag lists
+        static const SectionInfo sectionInfo_[]; //!< Exif (and one common Makernote) sections
 
     }; // class ExifTags
 
@@ -444,17 +437,38 @@ namespace Exiv2 {
     std::ostream& printDegrees(std::ostream& os, const Value& value);
     //! Print function converting from UCS-2LE to UTF-8
     std::ostream& printUcs2(std::ostream& os, const Value& value);
-
+    //! Print function for Exif units
+    std::ostream& printExifUnit(std::ostream& os, const Value& value);
+    //! Print GPS version
+    std::ostream& print0x0000(std::ostream& os, const Value& value);
+    //! Print GPS altitude ref
+    std::ostream& print0x0005(std::ostream& os, const Value& value);
     //! Print GPS altitude
     std::ostream& print0x0006(std::ostream& os, const Value& value);
     //! Print GPS timestamp
     std::ostream& print0x0007(std::ostream& os, const Value& value);
+    //! Print GPS status
+    std::ostream& print0x0009(std::ostream& os, const Value& value);
+    //! Print GPS measurement mode
+    std::ostream& print0x000a(std::ostream& os, const Value& value);
+    //! Print GPS speed ref
+    std::ostream& print0x000c(std::ostream& os, const Value& value);
+    //! Print GPS destination distance ref
+    std::ostream& print0x0019(std::ostream& os, const Value& value);
+    //! Print GPS differential correction
+    std::ostream& print0x001e(std::ostream& os, const Value& value);
+    //! Print orientation
+    std::ostream& print0x0112(std::ostream& os, const Value& value);
+    //! Print YCbCrPositioning
+    std::ostream& print0x0213(std::ostream& os, const Value& value);
     //! Print the copyright
     std::ostream& print0x8298(std::ostream& os, const Value& value);
     //! Print the exposure time
     std::ostream& print0x829a(std::ostream& os, const Value& value);
     //! Print the f-number
     std::ostream& print0x829d(std::ostream& os, const Value& value);
+    //! Print exposure program
+    std::ostream& print0x8822(std::ostream& os, const Value& value);
     //! Print ISO speed ratings
     std::ostream& print0x8827(std::ostream& os, const Value& value);
     //! Print components configuration specific to compressed data
@@ -467,14 +481,50 @@ namespace Exiv2 {
     std::ostream& print0x9204(std::ostream& os, const Value& value);
     //! Print the subject distance
     std::ostream& print0x9206(std::ostream& os, const Value& value);
+    //! Print metering mode
+    std::ostream& print0x9207(std::ostream& os, const Value& value);
+    //! Print light source
+    std::ostream& print0x9208(std::ostream& os, const Value& value);
     //! Print the actual focal length of the lens
     std::ostream& print0x920a(std::ostream& os, const Value& value);
     //! Print the user comment
     std::ostream& print0x9286(std::ostream& os, const Value& value);
+    //! Print color space
+    std::ostream& print0xa001(std::ostream& os, const Value& value);
+    //! Print sensing method
+    std::ostream& print0xa217(std::ostream& os, const Value& value);
+    //! Print file source
+    std::ostream& print0xa300(std::ostream& os, const Value& value);
+    //! Print scene type
+    std::ostream& print0xa301(std::ostream& os, const Value& value);
+    //! Print custom rendered
+    std::ostream& print0xa401(std::ostream& os, const Value& value);
+    //! Print exposure mode
+    std::ostream& print0xa402(std::ostream& os, const Value& value);
+    //! Print white balance
+    std::ostream& print0xa403(std::ostream& os, const Value& value);
     //! Print digital zoom ratio
     std::ostream& print0xa404(std::ostream& os, const Value& value);
     //! Print 35mm equivalent focal length
     std::ostream& print0xa405(std::ostream& os, const Value& value);
+    //! Print scene capture type
+    std::ostream& print0xa406(std::ostream& os, const Value& value);
+    //! Print gain control
+    std::ostream& print0xa407(std::ostream& os, const Value& value);
+    //! Print saturation
+    std::ostream& print0xa409(std::ostream& os, const Value& value);
+    //! Print subject distance range
+    std::ostream& print0xa40c(std::ostream& os, const Value& value);
+    //! Print GPS direction ref
+    std::ostream& printGPSDirRef(std::ostream& os, const Value& value);
+    //! Print contrast, sharpness (normal, soft, hard)
+    std::ostream& printNormalSoftHard(std::ostream& os, const Value& value);
+    //! Print any version packed in 4 Bytes format : major major minor minor
+    std::ostream& printExifVersion(std::ostream& os, const Value& value);
+    //! Print any version encoded in the ASCII string majormajorminorminor
+    std::ostream& printXmpVersion(std::ostream& os, const Value& value);
+    //! Print a date following the format YYYY-MM-DDTHH:MM:SSZ
+    std::ostream& printXmpDate(std::ostream& os, const Value& value);
     //@}
 
     //! Calculate F number from an APEX aperture value
